#include "wppostgraphicslistitem.h"
#include "WPData.h"
#include "themedatastorage.h"

#include <QPoint>
#include <QRect>
#include <QPainter>

// static member
WPPostGraphicsListItem::Geometry WPPostGraphicsListItem::iGeometry;

WPPostGraphicsListItem::WPPostGraphicsListItem(WPPost aPost, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : WPGraphicsListItem(aBusyIndicator, aThemeData)
    , iPost(aPost)
{
    setObjectName(QString::fromLatin1("WPPostListItem"));
    iDate = aPost->GetShortLocalTimeString();

    int commentCount = iPost->GetComments().count();

    if (commentCount == 1) {
        iCommentsText = tr("1 comment");
    } else {
        iCommentsText = tr("%1 comments") .arg(commentCount);
    }
}

WPPostGraphicsListItem::~WPPostGraphicsListItem()
{
}

bool WPPostGraphicsListItem::operator<(WPPostGraphicsListItem const & other)
{
    return (iPost->GetLocalTime() <other.iPost->GetLocalTime()); 
}

WPPost WPPostGraphicsListItem::GetPost() const
{
    return iPost;
}

void WPPostGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
    painter->setPen(Qt::black);

    painter->setFont(*iThemeData.iListTitleFont);
    painter->drawText(iGeometry.iTitle, iTitle);

    painter->setFont(*iThemeData.iListInfoFont);
    painter->drawText(iGeometry.iAuthor, "by " + iPost->iWpAuthorDisplayName);

    painter->setFont(*iThemeData.iListDetailFont);
    painter->drawText(iGeometry.iDate, iDate);
    painter->drawText(iGeometry.iCommentInfo, iCommentsText);
}

bool WPPostGraphicsListItem::IsBusy()
{
    return iPost->IsBusy();
}

bool WPPostGraphicsListItem::HasError()
{
    return iPost->iError;
}

void WPPostGraphicsListItem::handleGeomeryChange()
{
    iGeometry.update(size(), iThemeData);
    
    calculateElidedTexts();

    setMinimumHeight(iGeometry.height());
}

void WPPostGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(*iThemeData.iListTitleFont);
    iTitle = titleFm.elidedText(iPost->iTitle, Qt::ElideRight, int(iGeometry.iTitle.width()));
}
        
void WPPostGraphicsListItem::Geometry::update (const QSizeF &size, const ThemeData &theme)
{
    if (iCachedSize == size)
        // fresh
        return;

    else
        iCachedSize = size;

    // XXX: recalc boundingRect by hand
    int itemPadding = theme.iListItemPadding;
    int padding = theme.iListItemShadowMargin + itemPadding;
    int width = size.width() - 2 * padding;
    
    // Font metrics
    QFontMetrics titleFont (*theme.iListTitleFont);
    QFontMetrics infoFont (*theme.iListInfoFont);
    QFontMetrics detailFont (*theme.iListDetailFont);

    iTitle.setRect(padding, padding, width, titleFont.height());
    iAuthor.setRect(padding, iTitle.bottom() + itemPadding, width, infoFont.height());
    iDate.setRect(padding, iAuthor.bottom() + 3 * itemPadding, width, detailFont.height());
    iCommentInfo.setRect(padding, iDate.bottom() + itemPadding, width, detailFont.height());

    iHeight = iCommentInfo.bottom() + padding;
}
